@@ -1,13 +1,15 @@
 ##' -------------------------------------------------------- #
 ##' Author:          Reto Buergin
-##' E-Mail:          reto.buergin@unige.ch, rbuergin@gmx.ch
-##' Date:            2015-20-01
+##' E-Mail:          rbuergin@gmx.ch
+##' Date:            2016-01-10
 ##'
 ##' Description:
 ##' General utility functions for the 'vcrpart' package.
 ##' most of the functions are not exported and documented inline.
 ##'
 ##' Functions:
+##' neglogLik2.glm:        compute the -2 times likelihood error of
+##'                        a 'glm' object.
 ##' addEmptyChar:          add empty spaces to a character
 ##' appendDefArgs:         over-write default arguments
 ##' deparseCall:           convert a 'call' into a 'character'
@@ -24,6 +26,13 @@
 ##'                         from vcrpart_formula.
 ##'
 ##' Last modifications:
+##' 2015-09-02: - modified 'vcrpart_formula_eta' for implementation of
+##'               gaussian mixed model.
+##'             - set for baseline model as default category-specific
+##'               effects. 
+##' 2015-06-01: corrected bug of 'vcrpart_formula' in cases where a
+##'             effect modifier has the variable name 'x'. Now I call
+##'             it 'fTerm'.
 ##' 2015-01-20: change defaults for random effect specification
 ##' 2014-11-10: exported function 'contr.wsum'
 ##' 2014-09-08: partial substitution of 'rep' by 'rep.int'
@@ -50,7 +59,6 @@
 ##' - 
 ##' -------------------------------------------------------- #
 
-
 ## --------------------------------------------------------- #
 ##' Add empty spaces to characters to have the same length.
 ##'
@@ -318,7 +326,7 @@ vcrpart_fitted <- function(object, ...) {
 ##' Constructs a formula for the linear predictor.
 ##'
 ##' @param x       a formula.
-##' @param family  the model family, e.g., cumulative()
+##' @param fit     the fitting function, e.g., olmm() or glm()
 ##' @param env     environment for evaluating the formula.
 ##'
 ##' @return A formula.
@@ -350,29 +358,42 @@ vcrpart_formula_eta <- function(x, family, env) {
       }
       
       subsCe <- rval %in% termFact[attr(terms, "specials")$ce]
-      ceTerms <- unlist(lapply(rval[subsCe], function(x) eval(parse(text = x))))
+      ceTerms <-
+          unlist(lapply(rval[subsCe], function(fTerm) eval(parse(text = fTerm))))
       
       if (x$type == "vc" && any(sapply(ceTerms, checkOperators)))
         stop("the ':', '*' and '%in%' operators are not allowed for the 'by' ",
              "argument in 'vc' terms.")
       
       subsGe <- rval %in% termFact[attr(terms, "specials")$ge]
-      geTerms <- unlist(lapply(rval[subsGe], function(x) eval(parse(text = x)))) 
-      
+      geTerms <-
+          unlist(lapply(rval[subsGe], function(fTerm) eval(parse(text = fTerm)))) 
+          
       if (x$type == "vc" && any(sapply(geTerms, checkOperators)))
         stop("the ':', '*' and '%in%' operators are not allowed for the 'by' ",
              "argument in 'vc' terms.")
+
       ## This code distinguishes between the models and their random
       ## effect specification. For the cumulative and the adjacent model,
       ## use global effects, and for the cumulative model use category
       ## specific effects
-      ## if (family$family %in% c("cumulative", "adjacent")) {
-      ##   geTerms <- c(geTerms, rval[!subsGe & !subsCe])
-      ## } else {
-      ##   ceTerms <- c(ceTerms, rval[!subsGe & !subsCe])
-      ## }
+      if (family$family %in% c("cumulative", "adjacent")) {
+          geTerms <- c(geTerms, rval[!subsGe & !subsCe])
+      } else if (family$family %in% c("gaussian")) {
+          if (any(subsCe) | any(subsGe))
+              warning("'ce()' and 'ge()' terms ",
+                      "are ignored if family = '",
+                      family$family, "'")
+          ceTerms <- c(ceTerms, geTerms, rval[!subsGe & !subsCe])
+          subsCe <- rep(TRUE, length(ceTerms))
+          subsGe <- rep(FALSE, length(rval))
+          geTerms <- NULL          
+      } else {
+          ceTerms <- c(ceTerms, rval[!subsGe & !subsCe])
+      }
+      ## old
+      ## geTerms <- c(geTerms, rval[!subsGe & !subsCe]) # ? better
       
-      geTerms <- c(geTerms, rval[!subsGe & !subsCe]) # ? better
       rval <- list(paste(ceTerms, collapse = "+"),
                    paste(geTerms, collapse = "+"))
 
@@ -383,6 +404,8 @@ vcrpart_formula_eta <- function(x, family, env) {
     }
     
     ## set intercepts
+    if (family$family %in% c("baseline", "gaussian"))
+        x$intercept <- "ce"    
     if (x$intercept == "none") {
       int <- list(switch(x$type, fe = "-1", re = "-1", vc = "1"),
                   switch(x$type, fe = "1", re = "-1", vc = "1"))
@@ -548,8 +571,8 @@ vcrpart_formula <- function(formula, family = cumulative(),
     if (feTerms$intercept == "none") {
       direct <- vcInt == "ce"
       direct <- direct &
-        unlist(lapply(vcTerms, function(x) !"(Intercept)" %in%
-                      eval(parse(text = x))$nuisance))
+        unlist(lapply(vcTerms, function(fTerm) !"(Intercept)" %in%
+                      eval(parse(text = fTerm))$nuisance))
       subs <- which(direct)[1L]
     } else {
       subs <- c()
@@ -561,7 +584,7 @@ vcrpart_formula <- function(formula, family = cumulative(),
       direct <- direct[ord]
       vcTerms <- vcTerms[ord]
     }
-    vcTerms <- lapply(vcTerms, function(x) eval(parse(text = x)))
+    vcTerms <- lapply(vcTerms, function(fTerm) eval(parse(text = fTerm)))
     names(vcTerms) <- LETTERS[1:length(vcTerms)]
     for (pid in seq_along(vcTerms)) {
       vcTerms[[pid]]$name <- names(vcTerms)[pid]
